@@ -49,6 +49,7 @@ from . import examples
 from . import utilities
 from . import smallanglescattering
 from . import mpl
+from . import cloudscattering
 from . import test
 from .formel import loglist
 
@@ -59,6 +60,7 @@ sf = structurefactor
 ff = formfactor
 sas = smallanglescattering
 mplot = mpl.mplot
+cloud = cloudscattering
 
 from . import graceplot
 
