@@ -133,7 +133,7 @@ except ImportError:
     useFortran = False
 
 try:
-    import pymatgen
+    import pymatgen.core
 
     pymatgenfound = True
 except ImportError:
@@ -569,23 +569,25 @@ class lattice(object):
         else:
             self._points[choose, 3] = b
 
-    def alongLine(self, v, R, a=None, direction=0, b=1, invert=False):
+    def inCylinder(self, v, R, a=None, length=0, b=1, invert=False):
         """
         Set scattering length for points within a long cylinder.
 
         Parameters
         ----------
         a : 3 x float, default [0,0,0]
-            Point on cylinder axis.
+            Edge point on cylinder axis.
         R : float
             Radius of cylinder.
         v : 3 x float
             Normal vector along cylinder axis.
-        direction :
-            Which direction along normal vector.
-            - 0 both
-            - <0 negative direction
-            - >0 positive direction
+        length :
+            Length along cylinder axis.
+             - <0 negative direction
+             - >0 positive direction
+             - =0 infinite both directions
+             - np.inf positive direction, infinite length
+             - np.ninf negative direction, infinite length
         b:  float
             Scattering length for selected points.
         invert : bool
@@ -596,11 +598,15 @@ class lattice(object):
         ::
 
          import jscatter as js
-         sc=js.sf.scLattice(1,15,b=[0])
-         sc.alongLine(v=[1,1,1],a=[5,0,0],R=3)
-         sc.alongLine(v=[1,-1,1],a=[0,5,0],R=2)
-         sc.alongLine(v=[1,1,-1],a=[0,0,5],direction =1,R=1)
-         sc.alongLine(v=[1,0,0],a=[0,0,0],direction =-1,R=1)
+         import numpy as np
+         sc=js.sf.scLattice(0.5,30,b=[0])
+         sc.inCylinder(v=[1,1,1],a=[5,0,5],R=3)
+         sc.inCylinder(v=[1,-1,1],a=[0,5,0],R=2)
+         sc.inCylinder(v=[1,0,0],a=[0,-5,-10],length =10,R=2)
+         sc.inCylinder(v=[0,1,0],a=[5,0,-10],length =10,R=2)
+         sc.inCylinder(v=[1,0,0],a=[-10,-5,10],length =10,R=2,b=2)
+         sc.inCylinder(v=[0,1,0],a=[-5,0,10],length =-10,R=2,b=2)
+         sc.inCylinder(v=[1,0,0],a=[0,5,-10],length =np.NINF,R=2)
          sc.show()
 
         """
@@ -608,19 +614,24 @@ class lattice(object):
             a = [0, 0, 0]
         a = np.array(a)
         n = v / la.norm(v)
-        p = a - self._points[:, :3]
-        # np.dot is not thread safe
+        p = self._points[:, :3] - a
+        # distance from a  (np.dot is not thread safe)
         pn = np.einsum('ij,j', p, n)
+        # choose if in distance from axis
         choose = la.norm(p - (pn * n[:, None]).T, axis=1) < abs(R)
-        if direction > 0:
-            choose[pn > 0] = False
-        elif direction < 0:
-            choose[pn < 0] = False
+        # not choose if outside cylinder length
+        if length > 0:
+            choose[(pn < 0) | (pn > length)] = False
+        elif length < 0:
+            choose[(pn > 0) | (pn < length)] = False
 
         if invert:
             self._points[~choose, 3] = b
         else:
             self._points[choose, 3] = b
+        return
+
+    alongLine = inCylinder
 
     def show(self, R=None, cmap='rainbow', fig=None, ax=None, atomsize=1):
         """
@@ -757,7 +768,7 @@ class lattice(object):
 
 class pseudoRandomLattice(lattice):
 
-    def __init__(self, size, numberOfPoints, b=None, seed=None):
+    def __init__(self, size, numberOfPoints, unitCellAtoms=None, b=None, seed=None):
         """
         Create a  lattice with a pseudo random distribution of points.
 
@@ -766,15 +777,19 @@ class pseudoRandomLattice(lattice):
 
         Parameters
         ----------
-        size :3x float
+        size :list of 3x float
             Size of the lattice for each dimension relative to origin.
         numberOfPoints : int
             Number of points.
+        unitCellAtoms: list of 3x1 array, None=[0,0,0]
+            Analog to unit cell atoms but distributed around random point.
+            Position vectors vi of atoms in **absolute** units as *random_position+unitCellAtoms[i]*.
+            It is not checked if there is an overlap to other atoms.
         b : float,array
-            Scattering length of atoms. If array the sequence is repeated to fill N atoms.
+            Scattering length of atoms (in unitCellAtoms sequence).
         seed : None, int
             Seed for the Halton sequence by skipping the first seed elements of the sequence.
-            If None an random integer between 10 and 1e6 is chosen.
+            If None a random integer between 10 and 1e6 is chosen.
 
         Returns
         -------
@@ -789,29 +804,65 @@ class pseudoRandomLattice(lattice):
          grid=js.sf.pseudoRandomLattice([5,5,5],3000)
          fig=grid.show()
 
+         # three atom basis
+         grid=js.sf.pseudoRandomLattice([5,5,5],30,unitCellAtoms=[[0,0,0],[0.1,0.1,0],[0.3,0.3,0]],b=[1,2,3])
+         fig=grid.show()
+
+
         """
         # super(pseudoRandomLattice, self).__init__()
         super().__init__()
-        self.unitCellAtoms = [np.r_[0, 0, 0]]
+        if unitCellAtoms is None:
+            self.unitCellAtoms = [np.r_[0, 0, 0]]
+        else:
+            self.unitCellAtoms = unitCellAtoms
         if b is None:
             b = 1
         if isinstance(b, numbers.Number):
             b = [b]
+        if isinstance(b, numbers.Number):
+            b = [b] * len(unitCellAtoms)
         self.unitCellAtoms_b = b
-        self._makeLattice(size, numberOfPoints, b, seed)
+        self._size = size
+        if seed is None:
+            seed = np.random.randint(10, 1000000)
+        self._seed = seed
+        self._points = self._makeLattice(numberOfPoints, b, seed)
         self._type = 'pseudorandom'
 
-    def _makeLattice(self, size, N, pb=None, skip=None):
-        dim = np.shape(size)[0]
-        if skip is None:
-            skip = np.random.randint(10, 1000000)
+    def _makeLattice(self, N, pb=None, skip=None):
+        dim = np.shape(self._size)[0]
+        abc = formel.randomPointsInCube(N, skip=skip, dim=dim)
+        abc *= np.array(self._size)
+        # append missing dimension and b column
+        abc = np.c_[abc, np.zeros((N, 4 - dim))]
 
-        seq = formel.randomPointsInCube(N, skip=skip, dim=dim)
-        seq *= np.array(size)
-        if dim in [1, 2]:
-            seq = np.c_[seq, np.zeros((N, 3 - dim))]
-        self._points = np.c_[seq, np.tile(pb, N)[:N]]
+        # build lattice with all atoms in full grid with b
+        points = np.vstack([abc + np.r_[ev, b] for ev, b in
+                                  zip(self.unitCellAtoms, self.unitCellAtoms_b)])
+        return points
 
+    def appendPoints(self, N, b=None):
+        """
+        Add points to pseudorandom lattice. In place.
+
+        The Halton sequence is used with skipping the existing points.
+
+        Parameters
+        ----------
+        N : int
+            Number of points.
+        b : float,array
+            Scattering length of atoms. If array the sequence is repeated to fill N atoms.
+
+        """
+        if b is None:
+            b = 1
+        if isinstance(b, numbers.Number):
+            b = [b]
+        newpoints = self._makeLattice(N=N, pb=b, skip=self._seed + self._points.shape[0])
+        self._points = np.vstack([self._points, newpoints])
+        return
 
 class randomLattice(lattice):
 
@@ -1034,6 +1085,12 @@ class rhombicLattice(lattice):
              corresponding structure factor fhkl**2>0   [:, 3]
              corresponding indices hkl                  [:,4:]
 
+        Notes
+        -----
+        The threshold for selection rules allows to exclude forbidden peaks but include these for unit cells
+        with not equal scattering length as in these cases the selection rule not applies to full extend.
+        This is prefered over a explicit list of selection rules.
+
         """
 
         if isinstance(size, numbers.Number):
@@ -1138,20 +1195,41 @@ class rhombicLattice(lattice):
         vhkl = h * self.latticeVectors[0] + k * self.latticeVectors[1] + l * self.latticeVectors[2]
         return vhkl
 
-    def rotatePlane2hkl(self, plane, hkl):
+    def rotatePlane2hkl(self, plane, hkl, basis=None):
         """
         Rotate plane points that plane is perpendicular to hkl direction.
 
         Parameters
         ----------
         plane : array Nx3
-            3D points of plane
+            3D points of plane to rotate.
+            If None the rotation matrix is returned.
         hkl : list of int, float
             Miller indices as [1,1,1] indicating the lattice direction where to rotate to.
+        basis : array min 3x3, default=None
+            3 basis points spanning the plane to define rotation.
+            e.g.[[0,0,0],[1,0,0],[0,1,0]] for xy plane.
+            If None the first points of the plane are used instead.
+
 
         Returns
         -------
-            plane points array 3xN
+            rotated plane points array 3xN
+            or rotation matrix 3x3
+
+        Notes
+        -----
+        The rotation matrix may be used to rotate the plane  to the desired direction. ::
+
+         R = grid.rotatePlane2hkl(None,[1,1,1],[[0,0,0],[1,0,0],[0,1,0]] )
+         newplanepoints = np.einsum('ij,kj->ki', R, planepoints)
+
+        or the transposed R can be used to rotate the lattice. ::
+
+         R = grid.rotatePlane2hkl(None,[1,1,1],[[0,0,0],[1,0,0],[0,1,0]] )
+         grid.rotatebyMatrix(R.T)
+
+
 
         Examples
         --------
@@ -1170,15 +1248,21 @@ class rhombicLattice(lattice):
 
 
         """
+        if plane is not None:
+            plane = np.array(plane)
+        if basis is None:
+            basis = plane
+        else:
+            basis = np.array(basis)
         # hkl direction
         vhkl = self.vectorhkl(hkl)
         vhkl = vhkl /la.norm(vhkl)
         # search for vector v3 perpendicular to plane
         # first vector in grid close to point, then next with cross >0
-        v1 = plane[0] - plane[1]
+        v1 = basis[1] - basis[0]
         i = 2
         while True:
-            v2 = plane[0] - plane[i]
+            v2 = basis[i] - basis[0]
             v3 = np.cross(v1, v2)
             # test if >0 then it is perpendicular to plane as v1not parallel to v2
             if la.norm(v3) > 0:
@@ -1190,12 +1274,18 @@ class rhombicLattice(lattice):
         rotvector = np.cross(vhkl, v3)
         if la.norm(rotvector) < 1e-8:
             # is parallel
-            return plane
-
-        angle = np.arccos(np.clip(np.dot(vhkl / la.norm(vhkl), v3 / la.norm(v3)), -1.0, 1.0))
-        R = formel.rotationMatrix(rotvector, -angle)
-        Rplane = np.einsum('ij,kj->ki', R, plane)
-        return Rplane
+            if plane is None:
+                return np.eye(3)
+            else:
+                return plane
+        else:
+            angle = np.arccos(np.clip(np.dot(vhkl / la.norm(vhkl), v3 / la.norm(v3)), -1.0, 1.0))
+            R = formel.rotationMatrix(rotvector, -angle)
+            print(angle,rotvector)
+            if plane is None:
+                return R
+            else:
+                return np.einsum('ij,kj->ki', R, plane)
 
     def rotatePlaneAroundhkl(self, plane, hkl, angle):
         """
@@ -1203,8 +1293,8 @@ class rhombicLattice(lattice):
 
         Parameters
         ----------
-        plane : array Nx3
-            3D points of plane
+        plane : array Nx3, None
+            3D points of plane. If None the rotation matrix is returned.
         hkl : list of int, float
             Miller indices as [1,1,1] indicating the lattice direction to rotate around.
         angle : float
@@ -1213,6 +1303,19 @@ class rhombicLattice(lattice):
         Returns
         -------
             plane points array 3xN
+        or  rotation matrix
+
+
+        Notes
+        -----
+        The rotation matrix may be used to rotate the plane to the desired direction. ::
+
+         R = grid.rotatePlaneAroundhkl(None,[1,1,1],1.234)
+         newplanepoints = np.einsum('ij,kj->ki', R, planepoints)
+
+        or the transposed R can be used to rotate the lattice. ::
+
+         grid.rotatebyMatrix(R.T)
 
         Examples
         --------
@@ -1236,8 +1339,11 @@ class rhombicLattice(lattice):
         vhkl = self.vectorhkl(hkl)
 
         R = formel.rotationMatrix(vhkl, angle)
-        Rplane = np.einsum('ij,kj->ki', R, plane)
-        return Rplane
+        if plane is None:
+            return R
+        else:
+            Rplane = np.einsum('ij,kj->ki', R, plane)
+            return Rplane
 
     def rotatehkl2Vector(self, hkl, vector):
         """
@@ -1278,16 +1384,7 @@ class rhombicLattice(lattice):
         rotvector = np.cross(vhkl, vv)
         angle = np.arccos(np.clip(np.dot(vhkl, vv), -1.0, 1.0))
         R = formel.rotationMatrix(rotvector, -angle)
-
-        # rotate lattice vectors
-        self.latticeVectors = list(np.einsum('ij,kj->ki', R, self.latticeVectors))
-        # rotate points
-        # recreating of points by _makeLattice() is a bit faster for small lattices (N<20)
-        # for larger it is about the same (N=100)
-        self._points[:, :3] = np.einsum('ij,kj->ki', R, self._points[:, :3])
-
-        # update reciprocal vectors
-        self._makeReciprocalVectors()
+        self.rotatebyMatrix(R)
 
     def rotateAroundhkl(self, hkl, angle=None, vector=None, hkl2=None):
         """
@@ -1347,12 +1444,28 @@ class rhombicLattice(lattice):
                 angle = np.arccos(np.clip(np.dot(vhkl, vv), -1.0, 1.0))
 
         R = formel.rotationMatrix(vhkl, angle)
-        # rotate points
-        self._points[:, :3] = np.einsum('ij,kj->ki', R, self._points[:, :3])
-        # rotate lattice vectors
-        self.latticeVectors = list(np.einsum('ij,kj->ki', R, self.latticeVectors))
-        # update reciprocal vectors to reflect rotation
-        self._makeReciprocalVectors()
+        self.rotatebyMatrix(R)
+
+    def rotatebyMatrix(self, R):
+        """
+        Rotate lattice by rotation matrix including reciprocal vectors (in place).
+
+        Parameters
+        ----------
+        R : 3x3 array
+            Rotation matrix.
+
+        """
+        # test if it is rotation matrix
+        if not (np.allclose(la.det(R),1) and np.allclose(R @ R.T, np.eye(3))):
+            raise TypeError('R is not a rotation matrix.')
+        else:
+            # rotate points
+            self._points[:, :3] = np.einsum('ij,kj->ki', R, self._points[:, :3])
+            # rotate lattice vectors
+            self.latticeVectors = list(np.einsum('ij,kj->ki', R, self.latticeVectors))
+            # update reciprocal vectors to reflect rotation
+            self._makeReciprocalVectors()
 
 
 class bravaisLattice(rhombicLattice):
@@ -1395,7 +1508,7 @@ class scLattice(bravaisLattice):
         Parameters
         ----------
         abc : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 3x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1437,7 +1550,7 @@ class bccLattice(rhombicLattice):
         Parameters
         ----------
         abc : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 3x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1480,7 +1593,7 @@ class fccLattice(rhombicLattice):
         Parameters
         ----------
         abc : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 3x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1526,7 +1639,7 @@ class diamondLattice(rhombicLattice):
         Parameters
         ----------
         abc : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 3x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1576,7 +1689,7 @@ class hexLattice(rhombicLattice):
         Parameters
         ----------
         ab,c : float
-            Point distance.
+            Lattice constant of unit cell.
             ab is distance in hexagonal plane, c perpendicular.
             For c/a = (8/3)**0.5 the hcp structure
         size : 3x integer, integer
@@ -1611,6 +1724,50 @@ class hexLattice(rhombicLattice):
         self._type = 'hex'
 
 
+class honeycombLattice(rhombicLattice):
+
+    def __init__(self, ab, c, size, b=None):
+        """
+        Honeycomb lattice e.g for graphene.
+
+        See rhombicLattice for methods.
+
+        Parameters
+        ----------
+        ab,c : float
+            Lattice constants of unit cell.
+            ab is distance between nearest neighbors in honeycomb plane, c perpendicular.
+        size : 3x integer, integer
+            A list of integers describing the size in direction of the respective latticeVectors.
+            Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
+            If one integer is given it is used for all 3 dimensions.
+        b : list of float
+            Corresponding scattering length of atoms in the unit cell.
+
+        Returns
+        -------
+        lattice object
+            .array  grid points as numpy array
+
+        Examples
+        --------
+        ::
+
+         import jscatter as js
+         grid=js.sf.honeycombLattice(1.,2,[2,2,2],[1,2])
+         grid.show(5)
+
+        """
+        latticeVectors = [ab/2 * np.r_[3,  3 ** 0.5, 0.],
+                          ab/2 * np.r_[3, -3 ** 0.5, 0.],
+                          np.r_[0., 0., c]]
+        unitCellAtoms = [np.r_[0, 0, 0], np.r_[2, 2, 0]/3]
+        if isinstance(size, numbers.Number):
+            size = [size] * 3
+        rhombicLattice.__init__(self, latticeVectors, size, unitCellAtoms, b=b)
+        self._type = 'hex'
+
+
 class hcpLattice(rhombicLattice):
 
     def __init__(self, ab, size, b=None):
@@ -1622,7 +1779,7 @@ class hcpLattice(rhombicLattice):
         Parameters
         ----------
         ab : float
-            Point distance.
+            Lattice constant of unit cell.
             ab is distance in hexagonal plane, c = ab* (8/3)**0.5
         size : 3x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
@@ -1669,7 +1826,7 @@ class sqLattice(bravaisLattice):
         Parameters
         ----------
         ab : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 2x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1711,7 +1868,7 @@ class hex2DLattice(bravaisLattice):
         Parameters
         ----------
         ab : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 2x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1729,7 +1886,7 @@ class hex2DLattice(bravaisLattice):
         ::
 
          import jscatter as js
-         grid=js.sf.hexLattice(1.2,1)
+         grid=js.sf.hex2DLattice(1.2,1)
          grid.show(2)
 
         """
@@ -1753,7 +1910,7 @@ class lamLattice(bravaisLattice):
         Parameters
         ----------
         a : float
-            Point distance.
+            Lattice constant of unit cell.
         size : 1x integer, integer
             A list of integers describing the size in direction of the respective latticeVectors.
             Size is symmetric around zero in interval [-i,..,i] with length 2i+1.
@@ -1863,7 +2020,7 @@ class latticeFromCIF(rhombicLattice):
         #
         if isinstance(structure, str):
             if pymatgenfound:
-                cif = pymatgen.Structure.from_file(structure)
+                cif = pymatgen.core.Structure.from_file(structure)
             else:
                 raise Exception('Please install pymatgen to read CIF files. See doc of latticeFromCIF')
         else:
